/*	comm.cpp -- serial communication
*
*	Written by Cathy Saxton
*	cathy@idleloop.com
*/

#include <avr\io.h>
#include "abc.h"
#include "debug.h"
#include "comm.h"

/* REVIEW: disable if not DEBUG... */

char COMM::m_fInit = fFalse;

COMM::COMM()
{
	if (!m_fInit)
		Init();
}

/* do initialization for serial communication
   when 1MHz: we use 4800-8-N-1
   when 8MHz: we use 38400-8-N-1
   (just transmit at this time) */
void COMM::Init() const
{
	sbi(UCSRB, TXEN);	// enable transmit
		/* (UCSZ2 in UCSRB = 0 for 8-bit) */

#ifdef CLK_1MHZ
	/* 4800 baud is the fastest recommended for 1MHz
	   oscillator frequency; set with UBRR = 12 */
	UBRRH = 0;
	UBRRL = 12;
#else
	/* for 8Hmz, UBRR = 12 gives us 38.4 baud */
	UBRRH = 0;
	UBRRL = 12;
#endif

	/* set UCSRC (not UBRRH), asynch, no parity, 1 stop bit, 8 data bits */
	UCSRC = (1<<URSEL) | (1<<UCSZ1) | (1<<UCSZ0);

	m_fInit = fTrue;
}

#ifdef RX	// also would need to set RXEN in UCSRB
	while (!(UCSRA & (1 << RXC)))
		;
	ch = UDR;
	Blink(1);
#endif

void COMM::TxCh(char ch) const
{
	while (bit_is_clear(UCSRA, UDRE))
		;
	UDR = ch;
#ifdef NO
	while (bit_is_clear(UCSRA, TXC))	// wait for it to send
		;
#endif
}

void COMM::TxSz(const char *psz) const
{
	const char *pch;

	for (pch = psz; *pch != 0; ++pch)
		TxCh(*pch);
}

void COMM::TxSzCRLF(const char *psz) const
{
	TxSz(psz);
	TxCRLF();
}

void COMM::TxNum(unsigned long val) const
{
	char ach[16];
	char *pch, *pchMax;

	if (val == 0)
	{
		TxCh('0');
	}
	else
	{
		pch = (pchMax = ach + sizeof(ach)) - 1;
		while (val > 0)
		{
			*pch-- = '0' + val % 10;
			val /= 10;
		}
		while (++pch < pchMax)
			TxCh(*pch);
	}
}

void COMM::TxNumSigned(long val) const
{
	char ach[16];
	char *pch, *pchMax;

	if (val == 0)
	{
		TxCh('0');
	}
	else
	{
		if (val < 0)
		{
			TxCh('-');
			val = -val;
			if (val == 0)	// can't express pos in 32 bits
				TxSz("MAX");
		}
		pch = (pchMax = ach + sizeof(ach)) - 1;
		while (val > 0)
		{
			*pch-- = '0' + val % 10;
			val /= 10;
		}
		while (++pch < pchMax)
			TxCh(*pch);
	}
}

void COMM::TxByte(uchar val) const
{
	char ach[3];
	int ich, ichT;

	ich = 0;
//	if (val > 100)
		ach[ich++] = '0' + val / 100;
//	if (val > 10)
		ach[ich++] = '0' + (val % 100) / 10;
	ach[ich++] = '0' + (val % 10);

	for (ichT = 0; ichT < ich; ++ichT)
		TxCh(ach[ichT]);
}

void COMM::TxByteHex(uchar val) const
{
	TxDigitHex(val >> 4);
	TxDigitHex(val & 0x0f);
}

void COMM::TxDigitHex(uchar val) const
{
	Assert(val < 16);
	if (val <= 9)
		TxCh('0' + val);
	else
		TxCh('A' + (val - 10));
}

void COMM::TxSpc() const
{
	TxCh(' ');
}

void COMM::TxCRLF() const
{
	TxCh('\r');
	TxCh('\n');
}
