/*	eyes.cpp
*
*	Written by Cathy Saxton
*	cathy@idleloop.com
*/

#include <avr\io.h>
#include <avr\interrupt.h>	// for cli/sei
#include "abc.h"
#include "debug.h"
#include "eyes.h"

#define cread 3	// number of times to read sensor (for avg)

/* NOTE: keep in sync with ls enum in eyes.h */
ELS EYES::m_dnls[lsMax] =
{
	{aipLsFarLeft,  elsvBlackFarLeft,  elsvWhiteFarLeft},
	{aipLsLeft,     elsvBlackLeft,     elsvWhiteLeft},
	{aipLsCenter,   elsvBlackCenter,   elsvWhiteCenter},
	{aipLsRight,    elsvBlackRight,    elsvWhiteRight},
	{aipLsFarRight, elsvBlackFarRight, elsvWhiteFarRight},
};

#ifdef DEBUG
/* NOTE: keep in sync with ls enum in eyes.h */
static const char *s_mplspsz[lsMax] =
{
	"far L  ",
	"left   ",
	"center ",
	"right  ",
	"far R  ",
};
#endif

/* typical readings...
	meter black
		237 237 230 231 236
	meter white
		140 111 082 093 131
*/

/* (using 1/3's [top picture])
       +-----+-----+-----+
       |     |     |     |
     white   |     |   black
             |     |
            off    on
            line  line

       +---+---+---+---+
       |   |       |   |
     white |       | black
           |       |
          off     on
          line    line
*/

EYES::EYES()
{
	SetLineValues();
}

void EYES::Read(uchar ls, const AI &ai) const
{
	ELS *pels;

	pels = &m_dnls[ls];
	pels->alsv[ilsvCur] = ai.Read(pels->aip);
}

char EYES::FLostLine(uchar ls) const
{
	const ELS *pels;

	pels = &m_dnls[ls];
	return pels->alsv[ilsvCur] <= pels->alsv[ilsvOffLine];
}

char EYES::FLostLine(uchar ls, const AI &ai) const
{
	ELS *pels;

	pels = &m_dnls[ls];
	pels->alsv[ilsvCur] = ai.Read(pels->aip);
	return pels->alsv[ilsvCur] <= pels->alsv[ilsvOffLine];
}

char EYES::FFoundLine(uchar ls) const
{
	const ELS *pels;

	pels = &m_dnls[ls];
	return pels->alsv[ilsvCur] >= pels->alsv[ilsvOnLine];
}

char EYES::FFoundLine(uchar ls, const AI &ai) const
{
	ELS *pels;

	pels = &m_dnls[ls];
	pels->alsv[ilsvCur] = ai.Read(pels->aip);
	return pels->alsv[ilsvCur] >= pels->alsv[ilsvOnLine];
}

short EYES::Dlsv(uchar lsA, uchar lsB) const
{
	/* cast to ushort so doesn't sign extend!!! */
	return (short)((ushort)m_dnls[lsA].alsv[ilsvCur])
		- (short)((ushort)m_dnls[lsB].alsv[ilsvCur]);
}


/* read all light sensors and store average reading
   in "black" eeprom memory; update On/OffLine values */
void EYES::MeterBlack(const AI &ai) const
{
	const ELS *pels, *pelsMax;
	EEPROM eeprom;

	for (pelsMax = (pels = m_dnls) + lsMax; pels < pelsMax; ++pels)
		eeprom.Write(pels->elsvBlack, LsvAvgReading(ai, pels->aip));
	SetLineValues();
}

/* read all light sensors and store average reading
   in "white" eeprom memory; update On/OffLine values */
void EYES::MeterWhite(const AI &ai) const
{
	EEPROM eeprom;
	const ELS *pels, *pelsMax;

	for (pelsMax = (pels = m_dnls) + lsMax; pels < pelsMax; ++pels)
		eeprom.Write(pels->elsvWhite, LsvAvgReading(ai, pels->aip));
	SetLineValues();
}

void EYES::SetLineValues() const
{
	ELS *pels, *pelsMax;
	EEPROM eeprom;
	uchar lsvBlack, lsvWhite, dlsv;
#ifdef DEBUG
	COMM comm;

	comm.TxSz("white off on black...");
	comm.TxCRLF();
#endif

	/* read black/white */
	for (pelsMax = (pels = m_dnls) + lsMax; pels < pelsMax; ++pels)
	{
		lsvBlack = eeprom.Read(pels->elsvBlack);
		lsvWhite = eeprom.Read(pels->elsvWhite);
//		Assert(lsvBlack > lsvWhite);
		dlsv = (lsvBlack - lsvWhite) / 3;
		pels->alsv[ilsvOffLine] = lsvWhite + dlsv;
		pels->alsv[ilsvOnLine] = lsvBlack - dlsv;
#ifdef DEBUG
		comm.TxSz("  ");
		comm.TxSz(s_mplspsz[pels - m_dnls]);
		comm.TxSz(" - ");
		comm.TxByte(lsvWhite); comm.TxSpc();
		comm.TxByte(pels->alsv[ilsvOffLine]); comm.TxSpc();
		comm.TxByte(pels->alsv[ilsvOnLine]); comm.TxSpc();
		comm.TxByte(lsvBlack); comm.TxCRLF();
#endif
	}
}

/* actually reads from specified analog input, does averaging, and stores */
uchar EYES::LsvAvgReading(const AI &ai, uchar aip) const
{
	ushort lsv;
	uchar i;

	/* get sensor reading */
	lsv = 0;
	for (i = 0; i < cread; ++i)
		lsv += ai.Read(aip);
	lsv /= cread;

	return (uchar)lsv;
}
